/*
  Copyright (C) 1999  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __TEMPMATRIX_HPP
#define __TEMPMATRIX_HPP

#include "binarydefs.hpp"

template <class X>
class tempmatrix {
public:
  typedef int32 index_t; // must also be able to hold xsize * ysize !

  tempmatrix ();
  tempmatrix (const tempmatrix &o);
  tempmatrix & operator= (const tempmatrix &o);
  ~tempmatrix ();
  tempmatrix (const index_t xs_, const index_t ys_);

  inline index_t xsize () const { return xs; };
  inline index_t ysize () const { return ys; };
  inline const X &get (const index_t x, const index_t y) const { return matrix[x+y*xs]; };
  inline X &get (const index_t x, const index_t y) { return matrix[x+y*xs]; };

  inline const X &operator() (const index_t x, const index_t y) const { return get(x,y); };
  inline X &operator() (const index_t x, const index_t y) { return get(x,y); };

  void resize (const index_t nx, const index_t ny);
  void fill (const X &v);

private:
  index_t xs, ys;
  X *matrix;
};

template <class X>
tempmatrix<X>::tempmatrix ()
  : xs(0), ys(0), matrix(new X[0])
{
};

template <class X>
tempmatrix<X> & tempmatrix<X>::operator= (const tempmatrix &o)
{
  resize(o.xs, o.ys);
  index_t count=xs*ys;
  for (index_t i=0;i<count;++i) matrix[i]=o.matrix[i];
  return *this;
};

template <class X>
tempmatrix<X>::~tempmatrix ()
{
  delete [] matrix;
};

template <class X>
tempmatrix<X>::tempmatrix (const index_t xs_, const index_t ys_)
  : xs(xs_), ys(ys_), matrix(new X [xs_*ys_])
{
};

template <class X>
void tempmatrix<X>::resize (const index_t nx, const index_t ny)
{
  if ((nx==xs) && (ny==ys)) return;
  delete [] matrix;
  xs=nx;
  ys=ny;
  matrix=new X [xs*ys];
};

template <class X>
void tempmatrix<X>::fill (const X &v)
{
  index_t count=xs*ys;
  for (index_t i=0;i<count;++i) matrix[i]=v;

};

#endif
