/*

  yyysendables - makes it possible to send ints of all sizes through yyybinbuff
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "yyysendables.hpp"

using namespace yyybinbuff;

// uint8
void appendtobinbuffer (const uint8 a, yyybinbuff::binbuff &b)
{
  binbuff::index pos = b.size();
  b.resize(pos+1);
  b[pos]=a;  
};

const yyybinbuff::binbuff::index readfrombinbuffer(uint8 &a, const binbuff &b, const yyybinbuff::binbuff::index pos)
{
  if (b.size()-pos<1) return -1;
  a=b[pos];
  return 1;
};

// int8
void appendtobinbuffer (const int8 a, yyybinbuff::binbuff &b)
{
  appendtobinbuffer ((uint8)a, b);
};

const yyybinbuff::binbuff::index readfrombinbuffer(int8 &a, const binbuff &b, const yyybinbuff::binbuff::index pos)
{
  return readfrombinbuffer ((uint8)a, b, pos);
};

// uint16
void appendtobinbuffer (const uint16 a, yyybinbuff::binbuff &b)
{
  binbuff::index pos = b.size();
  b.resize (pos+2);
  b[pos] = a >> 8;
  b[pos+1] = a & 255;
};

const yyybinbuff::binbuff::index readfrombinbuffer(uint16 &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  if (b.size()-pos<2) return -1;
  a = uint16(b[pos]) << 8;
  a |= b[pos+1];
  return 2;
};

// int16
void appendtobinbuffer (const int16 a, yyybinbuff::binbuff &b)
{
  appendtobinbuffer ((uint16)a, b);
};

const yyybinbuff::binbuff::index readfrombinbuffer(int16 &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  return readfrombinbuffer ((uint16)a, b, pos);
};

// uint32
void appendtobinbuffer (const uint32 a, yyybinbuff::binbuff &b)
{
  binbuff::index pos = b.size();
  b.resize (pos+4);
  b[pos] = a >> 24;
  b[pos+1] = (a >> 16) & 255;
  b[pos+2] = (a >> 8) & 255;
  b[pos+3] = a & 255;
};

const yyybinbuff::binbuff::index readfrombinbuffer(uint32 &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  if (b.size()-pos<4) return -1;
  a = uint32(b[pos]) << 24;
  a |= uint32(b[pos+1]) << 16;
  a |= uint32(b[pos+2]) << 8;
  a |= b[pos+3];
  return 4;
};

// int32
void appendtobinbuffer (const int32 a, yyybinbuff::binbuff &b)
{
  appendtobinbuffer ((uint32)a, b);
};

const yyybinbuff::binbuff::index readfrombinbuffer(int32 &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  return readfrombinbuffer ((uint32)a, b, pos);
};

// bool
void appendtobinbuffer (const bool a, yyybinbuff::binbuff &b)
{
  binbuff::index pos = b.size();
  b.resize(pos+1);
  b[pos] = a ? 1 : 0;
};

const yyybinbuff::binbuff::index readfrombinbuffer(bool &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos)
{
  if (b.size()-pos<1) return -1;
  a = (b[pos]==0) ? false : true;
  return 1;
};
